superstrict
framework brl.blitz
import brl.glmax2d
import brl.timer
import brl.math
import brl.linkedlist
import brl.random

type TVector
field x:float
field y:float
function create:TVector(x:float,y:float)
	local v:TVector = new TVector
	v.x = x
	v.y = y
	return v
endfunction
method add_x(dx:float)
	x :+ dx
endmethod
method add_y(dy:float)
	y :+ dy
endmethod
method multiply(scalar:float)
	x :* scalar
	y :* scalar
endmethod
endtype

type TPlayer
field x:float
field y:float
field width:int
field height:int
field rotation:int
field vector:TVector
field image:TImage
field dead:int
method new()
	width = 15
	height = 50
	rotation = 0
	vector = TVector.create(0,0)
	create_image()
	dead = 0
endmethod
method create_image()
	image = createimage(width,height)
	local sqr2:int = sqr(width^2/2)
	local sqr4:int = sqr(width^2/4)
	setrotation(45)
	setcolor(0,0,255)
	drawrect(width/2,0,sqr2,sqr2)
	setrotation(0)
	setcolor(255,0,0)
	drawrect(0,sqr4,width,height*9/10-sqr4)
	setcolor(255,100,0)
	drawrect(width/5,height*9/10,0.6*width,height/10)
	grabimage(image,0,0)
	midhandleimage(image)
endmethod
method set_position(xp:int,yp:int)
	x = xp
	y = yp
endmethod
method accelerate(d:int)
	vector.add_x(d*sin(rotation)*0.25)
	vector.add_y(d*cos(rotation)*0.25)
endmethod
method shoot()
	local xpos:float = x + sin(rotation) * height / 2.0
	local ypos:float = y - cos(rotation) * height / 2.0
	TShot.create(xpos,ypos,rotation)
endmethod
method update()
	if dead then return
	if keydown(KEY_LEFT) then rotation :- 5
	if keydown(KEY_RIGHT) then rotation :+ 5
	if keydown(KEY_UP) then accelerate(1)
	if keydown(KEY_DOWN) then accelerate(-1)
	if keyhit(KEY_SPACE) then shoot()
	x :+ vector.x
	y :- vector.y
	for local i:TMeteor = eachin TMeteor.list
		if imagescollide2(image,x,y,0,rotation,1,1,i.image,i.x,i.y,0,0,1,1) then
			i.explode()
			die()
		endif
	next
	if imagescollide2(image,x,y,0,rotation,1,1,planet.image,planet.x,planet.y,0,0,1,1) then
		die()
		planet.damage(TPlanet.DAMAGE_PLAYER)
	endif
endmethod
method draw()
	if dead then return
	local buf_rot:float = getrotation()
	setrotation(rotation)
	setcolor(255,255,255)
	drawimage(image,x,y)
	setrotation(buf_rot)
endmethod
method die()
	dead = 1
endmethod
endtype

type TShot
global list:TList = new TList
field link:TLink
field x:float
field y:float
field width:int
field height:int
field image:TImage
field rotation:int
field vector:TVector
function clear()
	list.clear()
endfunction
function update_all()
	for local i:TShot = eachin list
		i.update()
	next
endfunction
function draw_all()
	for local i:TShot = eachin list
		i.draw()
	next
endfunction
function create:TShot(x:float,y:float,rotation:int,speed:int=20)
	local s:TShot = new TShot
	s.x = x
	s.y = y
	s.rotation = rotation
	s.vector = TVector.create(speed*sin(rotation),speed*cos(rotation))
	return s
endfunction
method new()
	width = 5
	height = 15
	link = list.addlast(self)
	create_image()
endmethod
method create_image()
	local mr:int,mg:int,mb:int
	getclscolor(mr,mg,mb)
	setmaskcolor mr,mg,mb
	image = createimage(width+10,height+10)
	setcolor(mr,mg,mb)
	drawrect(0,0,width+10,height+10)
	setcolor(255,0,0)
	setalpha(0.2)
	drawoval(0,0,width+10,height+10)
	setalpha(1)
	drawoval(5,5,width,height)
	grabimage(image,0,0)
	midhandleimage(image)
endmethod
method update()
	x :+ vector.x
	y :- vector.y
	for local i:TMeteor = eachin TMeteor.list
		if imagescollide2(image,x,y,0,rotation,1,1,i.image,i.x,i.y,0,0,1,1) then
			i.explode()
			remove()
		endif
	next
	if imagescollide2(image,x,y,0,rotation,1,1,planet.image,planet.x,planet.y,0,0,1,1) then
		remove()
		planet.damage(TPlanet.DAMAGE_SHOT)
	endif
	if x > 2*GW or x < -GW or y > 2*GH or y < -GH then remove()
endmethod
method draw()
	local buf_rot:float = getrotation()
	setcolor(255,255,255)
	setrotation(rotation)
	drawimage(image,x,y)
	setrotation(buf_rot)
endmethod
method remove()
	link.remove()
endmethod
endtype

type TMeteor
const TIME_AFTER_EXPLOSION:int = 2000
global list:TList = new TList
global exploded_list:TList = new TList
global last_created:int
field link:TLink
field x:float
field y:float
field size:int
field vector:TVector
field image:TImage
field has_exploded:int
field explode_time:int
field explosion_fade:float
function clear()
	list.clear()
	exploded_list.clear()
endfunction
function update_all()
	for local i:TMeteor = eachin list
		i.update()
	next
	for local i:TMeteor = eachin exploded_list
		i.update()
	next
	if millisecs() - last_created > 4000 then create_random()
endfunction
function draw_all()
	for local i:TMeteor = eachin list
		i.draw()
	next
	for local i:TMeteor = eachin exploded_list
		i.draw()
	next
endfunction
function create_random:TMeteor()
	local direction:int = rand(1,4)
	local xmin:int,xmax:int, ymin:int,ymax:int
	select direction
	case 1  xmin = -200;   xmax = GW+200; ymin = -200;   ymax = -200
	case 2  xmin = GW+200; xmax = GW+200; ymin = -200;   ymax = GH+200
	case 3  xmin = -200;   xmax = GW+200; ymin = GH+200; ymax = GH+200
	case 4  xmin = -200;   xmax = -200;   ymin = -200;   ymax = GH+200
	endselect
	local x:int = rand(xmin,xmax)
	local y:int = rand(ymin,ymax)
	local size:int = rand(30,50)
	local vector:TVector = TVector.create(planet.x-x+rand(-2*planet.size,2*planet.size),planet.y-y+rand(-2*planet.size,2*planet.size))
	vector.multiply(rnd(0.001,0.003))
	last_created = millisecs()
	return create(x,y,size,vector)
endfunction
function create:TMeteor(x:float,y:float,size:int,vector:TVector)
	local m:TMeteor = new TMeteor
	m.x = x
	m.y = y
	m.size = size
	m.vector = vector
	m.create_image()
	return m
endfunction
method new()
	link = list.addlast(self)
	has_exploded = 0
endmethod
method create_image()
	local mr:int, mg:int,mb:int
	getclscolor(mr,mg,mb)
	setmaskcolor(mr,mg,mb)
	image = createimage(size,size)
	setcolor(mr,mg,mb)
	drawrect(0,0,size,size)
	setcolor(71,36,0)
	drawoval(0,0,size,size)
	grabimage(image,0,0)
	midhandleimage(image)
endmethod
method update()
	if has_exploded then
		local dt:int = millisecs() - explode_time
		if dt > TIME_AFTER_EXPLOSION then remove()
		explosion_fade = 1 - float(dt)/TIME_AFTER_EXPLOSION
	else
		if imagescollide(image,x,y,0,planet.image,planet.x,planet.y,0) then
			explode()
			planet.damage(TPlanet.DAMAGE_METEOR)
		endif
		x :+ vector.x
		y :+ vector.y
	endif
	if x > 2*GW or x < -GW or y > 2*GH or y < -GH then remove()
endmethod
method draw()
	if has_exploded then
		setalpha(explosion_fade)
		setcolor(255,150,0)
		sethandle(size*0.75,size*0.75)
		drawoval(x,y,size*1.5,size*1.5)
		setalpha(1)
		sethandle(0,0)
	else
		setcolor(255,255,255)
		drawimage(image,x,y)
	endif
endmethod
method explode()
	has_exploded = 1
	explode_time = millisecs()
	link.remove()
	link = exploded_list.addlast(self)
endmethod
method remove()
	link.remove()
endmethod
endtype

type TPlanet
const DEVELOPMENT_PER_MS:float = 0.00001
const DAMAGE_METEOR:float = 0.2
const DAMAGE_SHOT:float = 0.1
const DAMAGE_PLAYER:float = 0.1
field x:int
field y:int
field size:int
field image:TImage
field status:float
field last_update:int
function create:TPlanet(size:int,status:float=0.25)
	local p:TPlanet = new TPlanet
	p.size = size
	p.create_image()
	p.status = status
	return p
endfunction
method new()
	x = GW/2
	y = GH/2
	last_update = millisecs()
endmethod
method create_image()
	local mr:int,mg:int,mb:int
	getclscolor(mr,mg,mb)
	image = createimage(size,size)
	setcolor(mr,mg,mb)
	drawrect(0,0,size,size)
	setcolor(rand(255),rand(255),rand(50,255))
	drawoval(0,0,size,size)
	grabimage(image,0,0)
	midhandleimage(image)
endmethod
method damage(d:float)
	status :- d
endmethod
method update()
	local ms:int = millisecs()
	local dt:int = ms - last_update
	status :+ dt*DEVELOPMENT_PER_MS
	last_update = ms
endmethod
method draw()
	setcolor(255,255,255)
	drawimage(image,x,y)
	local start:int = GW/10
	local stop:int = start+8*start*status
	local length:float = GW*9/10
	for local i:int = start to stop
		setcolor((1-(i-start)/length)*255,((i-start)/length)*255,0)
		drawrect i,GH-100,1,50
	next
endmethod
endtype

function game:int(timer:TTimer,player:TPlayer,planet:TPlanet)
	while not (keyhit(KEY_ESCAPE) or appterminate())
		timer.wait()
		player.update()
		planet.update()
		TMeteor.update_all()
		TShot.update_all()
		if planet.status >= 1 then return 1
		if planet.status <= 0 then return 0
		cls()
		planet.draw()
		player.draw()
		TMeteor.draw_all()
		TShot.draw_all()
		flip()
	wend
	end
endfunction

const GW:int = 1000
const GH:int = 700

seedrnd millisecs()
graphics(GW,GH,0)
setblend(ALPHABLEND)

global planet:TPlanet
local timer:TTimer = TTimer.create(40)

repeat
	local player:TPlayer = new TPlayer
	player.set_position(200,200)
	planet = TPlanet.create(200)
	TShot.clear()
	TMeteor.clear()

	local text:string = "Der Planet ist "
	if game(timer,player,planet) then
		text :+ "fertig entwickelt!"
	else
		text :+ "unbewohnbar. Und DU bist schuld."
	endif
	setcolor(255,255,255)
	sethandle(textwidth(text)/2,textheight(text)/2)
	drawtext text,GW/2,GH/2
	flip()
	sethandle(0,0)
	delay 3000
forever

